@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77info.c,v 1.54 2007/06/14 20:19:28 guru Exp $
+ *	$Id: mgd77info.c,v 1.66 2009/07/04 06:31:59 guru Exp $
  *
- *    Copyright (c) 2004-2007 by P. Wessel
+ *    Copyright (c) 2004-2009 by P. Wessel
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -26,12 +26,13 @@
 
 int main (int argc, char **argv)
 {
-	int i, id, rec, argno, col_listing = 3, length, id_col, t_col, x_col, y_col;
-	int header_flag = 0, n_paths, counter[MGD77_MAX_COLS], agency_listing = 1;
+	GMT_LONG i, id, rec, argno, col_listing = 3, length, id_col, t_col, x_col, y_col, saved_range, use;
+	GMT_LONG header_flag = 0, n_paths, counter[MGD77_MAX_COLS], agency_listing = 1, quad_no, n_quad;
 	
 	BOOLEAN error = FALSE, greenwich = FALSE, quick_summary = FALSE, dump_raw_header = FALSE;
 	BOOLEAN first = TRUE, col_summary = FALSE, read_file, dump_formatted_header = FALSE;
 	BOOLEAN quick_agencies = FALSE, dump_e77_header = FALSE, dump_hist_header = FALSE;
+	BOOLEAN quad[4] = {FALSE, FALSE, FALSE, FALSE};
 	
 	double this_dist, this_lon, this_lat, last_lon, last_lat, dx, dy, dlon, ds, lon_w;
 	double xmin, xmax, xmin1, xmin2, xmax1, xmax2, ymin, ymax, this_time, tmin, tmax;
@@ -45,6 +46,8 @@ int main (int argc, char **argv)
 	struct MGD77_DATASET *D;
 		
 	argc = GMT_begin (argc, argv);		/* Initialize GMT Machinery */
+	GMT_get_time_system ("unix", &(gmtdefs.time_system));						/* MGD77+ uses GMT's Unix time epoch */
+	GMT_init_time_system_structure (&(gmtdefs.time_system));
 	
 	/* Initialize MGD77 output order and other parameters*/
 	
@@ -201,12 +204,15 @@ int main (int argc, char **argv)
 	
 	read_file = (quick_summary || dump_raw_header);
 	
+	saved_range = GMT_io.geo.range;	/* We may have to reset thisso keep a copy */
 	GMT_io.out_col_type[0] = GMT_IS_LON;	GMT_io.out_col_type[1] = GMT_IS_LAT;
 	GMT_io.out_col_type[2] = M.time_format;	
-	if (quick_summary) printf ("#Cruise %sID      %sWest    %sEast    %sSouth   %sNorth   %sStartTime%s%sEndTime%s%s%sDist%snRec",
+	if (quick_summary) fprintf (GMT_stdout, "#Cruise %sID      %sWest    %sEast    %sSouth   %sNorth   %sStartTime%s%sEndTime%s%s%sDist%snRec",
 		gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter,
 		gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter);
 
+	use = (M.original || M.format != MGD77_FORMAT_CDF) ? MGD77_ORIG : MGD77_REVISED;
+	
 	for (argno = 0; argno < n_paths; argno++) {		/* Process each ID */
 	
 		if (MGD77_Open_File (list[argno], &M, MGD77_READ_MODE)) continue;
@@ -225,22 +231,22 @@ int main (int argc, char **argv)
 		}
 
 		if (dump_hist_header) {	/* Dump of MGD77+ history */
-			printf ("%s: %s", list[argno], D->H.history);
+			fprintf (GMT_stdout, "%s: %s", list[argno], D->H.history);
 			MGD77_Close_File (&M);
 			MGD77_Free (D);
 			continue;
 		}
 		if (dump_e77_header) {	/* Dump of e77 header status */
 			if (D->H.E77 && strlen(D->H.E77) > 0)
-				printf ("%s: %s\n", list[argno], D->H.E77);
+				fprintf (GMT_stdout, "%s: %s\n", list[argno], D->H.E77);
 			else
-				printf ("%s: E77 not applied\n", list[argno]);
+				fprintf (GMT_stdout, "%s: E77 not applied\n", list[argno]);
 			MGD77_Close_File (&M);
 			MGD77_Free (D);
 			continue;
 		}
 		if (dump_formatted_header) {	/* Dump of header items, one per line */
-			MGD77_Dump_Header_Params (&M, D->H.mgd77);	
+			MGD77_Dump_Header_Params (&M, D->H.mgd77[use]);	
 			MGD77_Close_File (&M);
 			MGD77_Free (D);
 			continue;
@@ -253,47 +259,46 @@ int main (int argc, char **argv)
 		if (first && quick_summary) {	/* Output all column headers */
 			for (i = 0; i < M.n_out_columns; i++) {
 				if (i == id_col || i == t_col || i == x_col || i == y_col) continue;
-				printf ("%s%s", gmtdefs.field_delimiter, D->H.info[M.order[i].set].col[M.order[i].item].abbrev);
+				fprintf (GMT_stdout,"%s%s", gmtdefs.field_delimiter, D->H.info[M.order[i].set].col[M.order[i].item].abbrev);
 			}
-			printf ("\n");
+			fprintf (GMT_stdout,"\n");
 		}
 		
 		if (col_summary) {	/* Just list names and info for any extra columns */
 			for (i = 0, first = TRUE; i < M.n_out_columns; i++) {
 				if (i == id_col || i == t_col || i == x_col || i == y_col) continue;
-				if (!first) printf ("%s", gmtdefs.field_delimiter);
+				if (!first) fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
 				if (((col_listing & 1) && M.order[i].set == 0) || ((col_listing & 2) && M.order[i].set == 1)) {
-					printf ("%s", D->H.info[M.order[i].set].col[M.order[i].item].abbrev);
+					fprintf (GMT_stdout, "%s", D->H.info[M.order[i].set].col[M.order[i].item].abbrev);
 					first = FALSE;
 				}
 			}
-			if (first) printf ("No columns matching selection found!");
-			printf ("\n");
+			if (first) fprintf (GMT_stdout, "No columns matching selection found!");
+			fprintf (GMT_stdout, "\n");
 			MGD77_Close_File (&M);
 			MGD77_Free (D);
 			continue;
 		}
 			
 		if (dump_raw_header) {	/* Write entire MGD77 header */
-			for (i = 0; i < 31; i++) putchar ('-');
-			printf (" Cruise: %8s ", M.NGDC_id);
-			for (i = 0; i < 31; i++) putchar ('-');
-			putchar ('\n');
+			fprintf (GMT_stdout, "-------------------------------");
+			fprintf (GMT_stdout, " Cruise: %8s ", M.NGDC_id);
+			fprintf (GMT_stdout, "-------------------------------\n");
 			MGD77_Write_Header_Record_m77 ("", &Out, &D->H);
-			for (i = 0; i < 80; i++) putchar ('-');
-			putchar ('\n');
+			fprintf (GMT_stdout, "----------------------------------------");
+			fprintf (GMT_stdout, "----------------------------------------\n");
 			if (M.format == MGD77_FORMAT_CDF) {
-				printf (D->H.history);
+				fprintf (GMT_stdout, "%s\n", D->H.history);
 				for (i = 0; i < M.n_out_columns; i++) {
 					if ((M.order[i].set == MGD77_CDF_SET)) {
-						printf ("> %s%s%s%s%s%s%s", D->H.info[MGD77_CDF_SET].col[M.order[i].item].abbrev, gmtdefs.field_delimiter,
+						fprintf (GMT_stdout, "> %s%s%s%s%s%s%s", D->H.info[MGD77_CDF_SET].col[M.order[i].item].abbrev, gmtdefs.field_delimiter,
 						D->H.info[MGD77_CDF_SET].col[M.order[i].item].name, gmtdefs.field_delimiter,
 						D->H.info[MGD77_CDF_SET].col[M.order[i].item].units, gmtdefs.field_delimiter,
 						D->H.info[MGD77_CDF_SET].col[M.order[i].item].comment);
 					}
 				}
 			}
-			putchar ('\n');
+			fprintf (GMT_stdout, "\n");
 		}
 		
 		GMT_make_dnan (tmin);
@@ -303,8 +308,9 @@ int main (int argc, char **argv)
 		xmax1 = xmax2 = -360.0;
 		ymin = 180.0;
 		ymax = -180.0;
+		memset ((void *)quad, 0, 4*sizeof(BOOLEAN));	/* Set al to FALSE */
 		greenwich = FALSE;
-		memset ((void *) counter, 0, (size_t)(MGD77_MAX_COLS * sizeof (int)));
+		memset ((void *) counter, 0, (size_t)(MGD77_MAX_COLS * sizeof (GMT_LONG)));
 	
 		for (i = 0; i < MGD77_MAX_COLS; i++) {
 			dvalue[i] = (double *)D->values[i];
@@ -329,7 +335,14 @@ int main (int argc, char **argv)
 			this_lon  = lon_w = dvalue[x_col][rec];
 			this_lat  = dvalue[y_col][rec];
 			if (this_lon < 0.0) this_lon += 360.0;	/* Start off with everything in 0-360 range */
-			if (lon_w >= 180.0) this_lon -= 360.0;	/* For -180/+180 range */
+			xmin1 = MIN (this_lon, xmin1);
+			xmax1 = MAX (this_lon, xmax1);
+			quad_no = (GMT_LONG)floor (this_lon/90.0);	/* Yields quadrants 0-3 */
+			if (quad_no == 4) quad_no = 0;		/* When this_lon == 360.0 */
+			quad[quad_no] = TRUE;
+			if (lon_w > 180.0) this_lon -= 360.0;	/* For -180/+180 range */
+			xmin2 = MIN (this_lon, xmin2);
+			xmax2 = MAX (this_lon, xmax2);
 			if (rec > 0) {	/* Need a previous point to calculate distance, speed, and heading */
 				dlon = this_lon - last_lon;
 				if (fabs (dlon) > 180.0) {
@@ -341,14 +354,6 @@ int main (int argc, char **argv)
 				ds = project_info.KM_PR_DEG * hypot (dx, dy);
 				this_dist += ds;
 			}
-			if (this_lon < 180.0) {
-				xmin1 = MIN (this_lon, xmin1);
-				xmax1 = MAX (this_lon, xmax1);
-			}
-			else {
-				xmin2 = MIN (lon_w, xmin2);
-				xmax2 = MAX (lon_w, xmax2);
-			}
 			ymin = MIN (this_lat, ymin);
 			ymax = MAX (this_lat, ymax);
 			
@@ -357,39 +362,53 @@ int main (int argc, char **argv)
 			for (i = 1; i < M.n_out_columns; i++) {
 				if (i == id_col || i == t_col || i == x_col || i == y_col) continue;
 				if ((length = D->H.info[M.order[i].set].col[M.order[i].item].text)) {
-					if (strncmp (&tvalue[i][rec*length], ALL_NINES, length)) counter[i]++;
+					if (strncmp (&tvalue[i][rec*length], ALL_NINES, (size_t)length)) counter[i]++;
 				}
 				else
 					if (!GMT_is_dnan (dvalue[i][rec])) counter[i]++;
 			}
 		}
 
-		if (xmin1 == 360.0) {	/* Only negative longitudes found */
+		GMT_io.geo.range = saved_range;	/* We reset this each time */
+		n_quad = quad[0] + quad[1] + quad[2] + quad[3];	/* How many quadrants had data */
+		if (quad[0] && quad[3]) {	/* Longitudes on either side of Greenwich only, must use -180/+180 notation */
 			xmin = xmin2;
 			xmax = xmax2;
+			GMT_io.geo.range = 2;	/* Override this setting explicitly */
 		}
-		else if (xmin2 == 360.0) {	/* Only positive longitudes found */
+		else if (quad[1] && quad[2]) {	/* Longitudes on either side of the date line, must user 0/360 notation */
+			xmin = xmin1;
+			xmax = xmax1;
+			GMT_io.geo.range = 0;	/* Override this setting explicitly */
+		}
+		else if (n_quad == 2 && ((quad[0] && quad[2]) || (quad[1] && quad[3]))) {	/* Funny quadrant gap, pick shortest longitude extent */
+			if ((xmax1 - xmin1) < (xmax2 - xmin2)) {	/* 0/360 more compact */
+				xmin = xmin1;
+				xmax = xmax1;
+				GMT_io.geo.range = 0;	/* Override this setting explicitly */
+			}
+			else {						/* -180/+180 more compact */
+				xmin = xmin2;
+				xmax = xmax2;
+				GMT_io.geo.range = 2;	/* Override this setting explicitly */
+			}
+		}
+		else {						/* Either will do, use default settings */
 			xmin = xmin1;
 			xmax = xmax1;
 		}
-		else if ((xmin1 - xmax2) < 90.0) {	/* Crossed Greenwich */
-			xmin = xmin2;
-			xmax = xmax1;
-		}
-		else {					/* Crossed Dateline */
-			xmin = xmin1;
-			xmax = xmax2;
-		}
+		if (xmin > xmax) xmin -= 360.0;
+		if (xmin < 0.0 && xmax < 0.0) xmin += 360.0, xmax += 360.0;
 
 		if (GMT_is_dnan(tmin) || GMT_is_dnan(tmax)) {
 			int yy[2], mm[2], dd[2];
 			if (gmtdefs.verbose) fprintf (stderr, "%s warning: cruise %s no time records.\n",GMT_program, M.NGDC_id);
-			yy[0] = (!D->H.mgd77->Survey_Departure_Year[0] || !strncmp (D->H.mgd77->Survey_Departure_Year, ALL_BLANKS, 4)) ? 0 : atoi (D->H.mgd77->Survey_Departure_Year);
-			yy[1] = (!D->H.mgd77->Survey_Arrival_Year[0] || !strncmp (D->H.mgd77->Survey_Arrival_Year, ALL_BLANKS, 4)) ? 0 : atoi (D->H.mgd77->Survey_Arrival_Year);
-			mm[0] = (!D->H.mgd77->Survey_Departure_Month[0] || !strncmp (D->H.mgd77->Survey_Departure_Month, ALL_BLANKS, 2)) ? 1 : atoi (D->H.mgd77->Survey_Departure_Month);
-			mm[1] = (!D->H.mgd77->Survey_Arrival_Month[0] || !strncmp (D->H.mgd77->Survey_Arrival_Month, ALL_BLANKS, 2)) ? 1 : atoi (D->H.mgd77->Survey_Arrival_Month);
-			dd[0] = (!D->H.mgd77->Survey_Departure_Day[0] || !strncmp (D->H.mgd77->Survey_Departure_Day, ALL_BLANKS, 2)) ? 1 : atoi (D->H.mgd77->Survey_Departure_Day);
-			dd[1] = (!D->H.mgd77->Survey_Arrival_Day[0] || !strncmp (D->H.mgd77->Survey_Arrival_Day, ALL_BLANKS, 2)) ? 1 : atoi (D->H.mgd77->Survey_Arrival_Day);
+			yy[0] = (!D->H.mgd77[use]->Survey_Departure_Year[0] || !strncmp (D->H.mgd77[use]->Survey_Departure_Year, ALL_BLANKS, (size_t)4)) ? 0 : atoi (D->H.mgd77[use]->Survey_Departure_Year);
+			yy[1] = (!D->H.mgd77[use]->Survey_Arrival_Year[0] || !strncmp (D->H.mgd77[use]->Survey_Arrival_Year, ALL_BLANKS, (size_t)4)) ? 0 : atoi (D->H.mgd77[use]->Survey_Arrival_Year);
+			mm[0] = (!D->H.mgd77[use]->Survey_Departure_Month[0] || !strncmp (D->H.mgd77[use]->Survey_Departure_Month, ALL_BLANKS, (size_t)2)) ? 1 : atoi (D->H.mgd77[use]->Survey_Departure_Month);
+			mm[1] = (!D->H.mgd77[use]->Survey_Arrival_Month[0] || !strncmp (D->H.mgd77[use]->Survey_Arrival_Month, ALL_BLANKS, (size_t)2)) ? 1 : atoi (D->H.mgd77[use]->Survey_Arrival_Month);
+			dd[0] = (!D->H.mgd77[use]->Survey_Departure_Day[0] || !strncmp (D->H.mgd77[use]->Survey_Departure_Day, ALL_BLANKS, (size_t)2)) ? 1 : atoi (D->H.mgd77[use]->Survey_Departure_Day);
+			dd[1] = (!D->H.mgd77[use]->Survey_Arrival_Day[0] || !strncmp (D->H.mgd77[use]->Survey_Arrival_Day, ALL_BLANKS, (size_t)2)) ? 1 : atoi (D->H.mgd77[use]->Survey_Arrival_Day);
 			if (! (yy[0] == 0 && yy[1] == 0)) {	/* With year we can do something */
 				rata_die = GMT_rd_from_gymd (yy[0], mm[0], dd[0]);
 				tmin = GMT_rdc2dt (rata_die, 0.0);
@@ -398,7 +417,7 @@ int main (int argc, char **argv)
 			}
 		}			
 		if (quick_summary) {
-			printf ("%8s%s%8s%s", M.NGDC_id, gmtdefs.field_delimiter, D->H.mgd77->Survey_Identifier, gmtdefs.field_delimiter);
+			fprintf (GMT_stdout,"%8s%s%8s%s", M.NGDC_id, gmtdefs.field_delimiter, D->H.mgd77[use]->Survey_Identifier, gmtdefs.field_delimiter);
 			GMT_ascii_output_one (GMT_stdout, xmin, 0);	fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
 			GMT_ascii_output_one (GMT_stdout, xmax, 0);	fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
 			GMT_ascii_output_one (GMT_stdout, ymin, 1);	fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
@@ -407,20 +426,23 @@ int main (int argc, char **argv)
 				GMT_ascii_output_one (GMT_stdout, tmin, 2);	fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
 				GMT_ascii_output_one (GMT_stdout, tmax, 2);	fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);						
 			} else {
-				printf ("%4s-%2s-%2s%s%4s-%2s-%2s%s",
-				D->H.mgd77->Survey_Departure_Year, D->H.mgd77->Survey_Departure_Month, D->H.mgd77->Survey_Departure_Day, gmtdefs.field_delimiter,
-				D->H.mgd77->Survey_Arrival_Year, D->H.mgd77->Survey_Arrival_Month, D->H.mgd77->Survey_Arrival_Day, gmtdefs.field_delimiter);
+				fprintf (GMT_stdout, "%4s-%2s-%2s%s%4s-%2s-%2s%s",
+				D->H.mgd77[use]->Survey_Departure_Year, D->H.mgd77[use]->Survey_Departure_Month, D->H.mgd77[use]->Survey_Departure_Day, gmtdefs.field_delimiter,
+				D->H.mgd77[use]->Survey_Arrival_Year, D->H.mgd77[use]->Survey_Arrival_Month, D->H.mgd77[use]->Survey_Arrival_Day, gmtdefs.field_delimiter);
 			}
-			printf ("%d%s%d", irint (this_dist), gmtdefs.field_delimiter, D->H.n_records);
+			fprintf (GMT_stdout, "%ld%s%ld", (GMT_LONG)irint (this_dist), gmtdefs.field_delimiter, D->H.n_records);
 			for (i = 1; i < M.n_out_columns; i++) {
 				if (i == id_col || i == t_col || i == x_col || i == y_col) continue;
 				if (((col_listing & 1) && M.order[i].set == 0) || ((col_listing & 2) && M.order[i].set == 1))
-					printf ("%s%d",	gmtdefs.field_delimiter, counter[i]);
+					fprintf (GMT_stdout,"%s%ld",	gmtdefs.field_delimiter, counter[i]);
 			}
-			printf ("\n");
+			fprintf (GMT_stdout,"\n");
 		}
 		MGD77_Free (D);
 	}
 		
+	MGD77_Path_Free (n_paths, list);
+	MGD77_end (&M);
+
 	exit (EXIT_SUCCESS);
 }
