@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77path.c,v 1.22 2009/01/09 04:02:35 guru Exp $
+ *	$Id: mgd77path.c,v 1.23 2009/06/23 20:26:58 guru Exp $
  *
  *    Copyright (c) 2004-2009 by P. Wessel
  *    See README file for copying and redistribution conditions.
@@ -21,7 +21,7 @@ int main (int argc, char **argv)
 {
 	GMT_LONG i, n_cruises = 0, n_paths;
 	
-	BOOLEAN error = FALSE, dirs = FALSE;
+	BOOLEAN error = FALSE, dirs = FALSE, strip = FALSE;
 	
 	char path[BUFSIZ], **list;
 	
@@ -49,6 +49,7 @@ int main (int argc, char **argv)
 
 			case 'P':	/* Show list of paths to MGD77 files */
 				dirs = FALSE;
+				if (argv[i][2] == '-') strip = TRUE;
 				break;
 
 			default:		/* Options not recognized */
@@ -59,13 +60,13 @@ int main (int argc, char **argv)
 
 	if (GMT_give_synopsis_and_exit || argc == 1) {	/* Display usage */
 		fprintf(stderr,"mgd77path %s - Return paths to MGD77 cruises and directories\n\n", MGD77_VERSION);
-		fprintf(stderr,"usage: mgd77path <cruise(s)> -D|P [-I<code>] [-V]\n\n");
+		fprintf(stderr,"usage: mgd77path <cruise(s)> -D|P[-] [-I<code>] [-V]\n\n");
          
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
               
 		MGD77_Cruise_Explain ();
 		fprintf(stderr,"	-D Instead of full cruise paths just list all directories with MGD77 files\n");
-		fprintf(stderr,"	-P List full cruise paths\n");
+		fprintf(stderr,"	-P List full cruise paths [Default].  Append - to only get cruise names\n");
 		fprintf(stderr,"	OPTIONS:\n\n");
 		fprintf(stderr,"	-I Ignore certain data file formats from consideration. Append combination of act to ignore\n");
 		fprintf(stderr,"	   (a) MGD77 ASCII, (c) MGD77+ netCDF, or (t) plain table files. [Default ignores none]\n");
@@ -90,6 +91,10 @@ int main (int argc, char **argv)
 	for (i = 0; i < n_paths; i++) {		/* Process each ID */
  		if (MGD77_Get_Path (path, list[i], &M))
    			fprintf (stderr, "%s : Cannot find cruise %s\n", GMT_program, list[i]);
+		else if (strip) {
+			printf ("%s\n", list[i]);
+			n_cruises++;
+		}
 		else {
 			printf ("%s\n", path);
 			n_cruises++;
