@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt2kml.c,v 1.18 2009/07/11 08:33:28 guru Exp $
+ *	$Id: gmt2kml.c,v 1.23 2009/09/14 02:11:05 guru Exp $
  *
  *	Copyright (c) 2009 by P. Wessel
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -47,12 +47,13 @@ struct EXT_COL {
 BOOLEAN check_lon_lat (double *lon, double *lat, double west, double east, double north, double south);
 void print_altmode (int extrude, int fmode, int altmode);
 void place_region_tag (double west, double east, double south, double north, double alt_min, double alt_max, GMT_LONG lod_min, GMT_LONG lod_max, double fade_min, double fade_max);
+GMT_LONG ascii_output_one (double x, GMT_LONG col);
 
 int main (int argc, char **argv)
 {
 	GMT_LONG i, j, k, fno, n_files = 0, n_args, fmode = POINT, altmode = KML_GROUND, n_coord = 0, itransp[5];
 	GMT_LONG ix, iy, t1_col, t2_col, pos, t_itransp, n_ext_cols = 0, n_alloc = 0, n_rec = 0;
-	GMT_LONG lod_min = 0, lod_max = 0, n, set_nr = 0, pnt_nr = 0, index = -4;
+	GMT_LONG lod_min = 0, lod_max = 0, set_nr = 0, pnt_nr = 0, index = -4;
 
 	BOOLEAN error = FALSE, long_verbose = FALSE, nofile = TRUE, done = FALSE, greenwich = FALSE, first = TRUE;
 	BOOLEAN get_label = FALSE, no_label = FALSE, visible = TRUE, open = FALSE, use_folder = FALSE;
@@ -61,7 +62,7 @@ int main (int argc, char **argv)
 	char doc_title[BUFSIZ], folder_name[BUFSIZ], icon[BUFSIZ];
 	char *feature[5] = {"Point", "Point", "Point", "LineString", "Polygon"}, *name[5] = {"Point", "Event", "Timespan", "Line", "Polygon"};
 	char *indent[5] = {"\t\t\t", "\t\t\t", "\t\t\t", "\t\t\t", "\t\t\t\t\t"}, *fmt = NULL, buffer[BUFSIZ], label[BUFSIZ];
-	char *Document[2] = {"Document", "Folder"}, p[BUFSIZ], C[4][GMT_TEXT_LEN];
+	char *Document[2] = {"Document", "Folder"}, p[BUFSIZ], C[4][GMT_TEXT_LEN], *c;
 
 	int rgb[3];
 
@@ -82,7 +83,7 @@ int main (int argc, char **argv)
 	GMT_init_fill (&fill, 255, 192, 128);		/* Default fill color */
 	GMT_init_fill (&t_fill, 255, 255, 255);		/* Default text color */
 	GMT_init_pen (&pen, 1.0);			/* Default pen width */
-	doc_title[0] = folder_name[0] = '\0';
+	doc_title[0] = folder_name[0] = label[0] = '\0';
 	strcpy (icon, "http://maps.google.com/mapfiles/kml/pal4/icon57.png");
 	
 	for (i = 1; i < argc; i++) {
@@ -271,9 +272,15 @@ int main (int argc, char **argv)
 						error++;
 					}
 					break;
-				case 'T':	/* Title */
-					n = sscanf (&argv[i][2], "%[^/]/%s", doc_title, folder_name);
-					if (n == 1) folder_name[0] = '\0';
+				case 'T':	/* Title [and folder] */
+					if ((c = strchr (&argv[i][2], '/'))) {	/* Got both title and folder */
+						strcpy (folder_name, &c[1]);
+						*c = '\0';
+						strcpy (doc_title, &argv[i][2]);
+						*c = '/';
+					}
+					else
+						strcpy (doc_title, &argv[i][2]);
 					break;
 				case 'W':
 					if (argv[i][2] == '-')
@@ -596,9 +603,9 @@ int main (int argc, char **argv)
 				printf ("\t\t<%s>\n", feature[fmode]);
 				print_altmode (extrude, FALSE, altmode);
 				printf ("\t\t\t<coordinates>");
-				GMT_ascii_output_one (GMT_stdout, out[GMT_X], GMT_X);	printf (",");
-				GMT_ascii_output_one (GMT_stdout, out[GMT_Y], GMT_Y);	printf (",");
-				GMT_ascii_output_one (GMT_stdout, out[GMT_Z], GMT_Z);
+				ascii_output_one (out[GMT_X], GMT_X);	printf (",");
+				ascii_output_one (out[GMT_Y], GMT_Y);	printf (",");
+				ascii_output_one (out[GMT_Z], GMT_Z);	printf ("\n");
 				printf ("</coordinates>\n");
 				printf ("\t\t</%s>\n", feature[fmode]);
 				printf ("\t</Placemark>\n");
@@ -640,7 +647,8 @@ int main (int argc, char **argv)
 					}
 					else {
 						printf ("\t<Placemark>\n");
-						if (line->segment[i]->label)
+						if (no_label) { /* Nothing */ }
+						else if (line->segment[i]->label)
 							printf ("\t\t<name>%s</name>\n", line->segment[i]->label);
 						else if (fmt) {
 							printf ("\t\t<name>"); printf (fmt, (int)set_nr); printf ("</name>\n");
@@ -677,7 +685,7 @@ int main (int argc, char **argv)
 							for (k = 0; k < n_ext_cols; k++) {
 								printf ("\t\t\t\t<Data name = \"%s\">\n", D[k].name);
 								printf ("\t\t\t\t\t<value>");
-								GMT_ascii_output_one (GMT_stdout, line->segment[i]->coord[D[k].col][j], D[k].col);
+								ascii_output_one (line->segment[i]->coord[D[k].col][j], D[k].col);
 								printf ("</value>\n\t\t\t\t</Data>\n");
 							}
 							printf ("\t\t\t</ExtendedData>>\n");
@@ -686,28 +694,28 @@ int main (int argc, char **argv)
 							printf ("\t\t\t<TimeSpan>\n");
 							if (!GMT_is_dnan(line->segment[i]->coord[t1_col][j])) {
 								printf ("\t\t\t\t<begin>");
-								GMT_ascii_output_one (GMT_stdout, line->segment[i]->coord[t1_col][j], t1_col);
+								ascii_output_one (line->segment[i]->coord[t1_col][j], t1_col);
 								printf ("</begin>\n");
 							}
 							if (!GMT_is_dnan(line->segment[i]->coord[t2_col][j])) {
 								printf ("\t\t\t\t<end>");
-								GMT_ascii_output_one (GMT_stdout, line->segment[i]->coord[t2_col][j], t2_col);
+								ascii_output_one (line->segment[i]->coord[t2_col][j], t2_col);
 								printf ("</end>\n");
 							}
 							printf ("\t\t\t</TimeSpan>\n");
 						}
 						else if (fmode == EVENT) {
 							printf ("\t\t\t<TimeStamp>\n\t\t\t\t<when>\n");
-							GMT_ascii_output_one (GMT_stdout, line->segment[i]->coord[t1_col][j], t1_col);
+							ascii_output_one (line->segment[i]->coord[t1_col][j], t1_col);
 							printf ("</when>\n\t\t\t</TimeStamp>\n");
 						}
 						printf ("\t\t\t<styleUrl>#GMT%ld</styleUrl>\n", index);
 						printf ("\t\t\t<%s>\n\t", feature[fmode]);
 						print_altmode (extrude, FALSE, altmode);
 						printf ("\t\t\t\t<coordinates>");
-						GMT_ascii_output_one (GMT_stdout, out[GMT_X], GMT_X);	printf (",");
-						GMT_ascii_output_one (GMT_stdout, out[GMT_Y], GMT_Y);	printf (",");
-						GMT_ascii_output_one (GMT_stdout, out[GMT_Z], GMT_Z);
+						ascii_output_one (out[GMT_X], GMT_X);	printf (",");
+						ascii_output_one (out[GMT_Y], GMT_Y);	printf (",");
+						ascii_output_one (out[GMT_Z], GMT_Z);	printf ("\n");
 						printf ("</coordinates>\n");
 						printf ("\t\t\t</%s>\n", feature[fmode]);
 						printf ("\t\t</Placemark>\n");
@@ -715,7 +723,9 @@ int main (int argc, char **argv)
 					else {
 						if (GMT_is_fnan(out[GMT_Z])) out[GMT_Z] = 0.0;	/* Google Earth can not handle lines at NaN altitude */
 						printf ("%s\t", indent[fmode]);
-						GMT_output (GMT_stdout, 3, out);
+						ascii_output_one (out[GMT_X], GMT_X);	printf (",");
+						ascii_output_one (out[GMT_Y], GMT_Y);	printf (",");
+						ascii_output_one (out[GMT_Z], GMT_Z);	printf ("\n");
 					}
 					pnt_nr++;
 				}
@@ -772,11 +782,11 @@ void place_region_tag (double west, double east, double south, double north, dou
 {
 	if (GMT_360_RANGE (west, east)) { west = -180.0; east = +180.0;}
 	printf ("\t<Region>\n\t\t<LatLonAltBox>\n");
-	printf ("\t\t\t<north>");		GMT_ascii_output_one (GMT_stdout, north, GMT_Y);
-	printf ("</north>\n\t\t\t<south>");	GMT_ascii_output_one (GMT_stdout, south, GMT_Y);
+	printf ("\t\t\t<north>");		ascii_output_one (north, GMT_Y);
+	printf ("</north>\n\t\t\t<south>");	ascii_output_one (south, GMT_Y);
 	printf ("</south>\n");
-	printf ("\t\t\t<east>");		GMT_ascii_output_one (GMT_stdout, east, GMT_X);
-	printf ("</east>\n\t\t\t<west>");	GMT_ascii_output_one (GMT_stdout, west, GMT_X);
+	printf ("\t\t\t<east>");		ascii_output_one (east, GMT_X);
+	printf ("</east>\n\t\t\t<west>");	ascii_output_one (west, GMT_X);
 	printf ("</west>\n");
 	if (alt_max > alt_min) {
 		printf ("\t\t\t<minAltitude>%g</minAltitude>\n", alt_min);
@@ -795,3 +805,11 @@ void place_region_tag (double west, double east, double south, double north, dou
 	}
 	printf ("\t</Region>\n");
 }
+
+GMT_LONG ascii_output_one (double x, GMT_LONG col)
+{	/* Used instead of GMT_asfii_output_one since Windoze has trouble with GMT_stdout and stdout mixing */
+	char text[GMT_LONG_TEXT];
+
+	GMT_ascii_format_one (text, x, GMT_io.out_col_type[col]);
+	return (printf ("%s", text));
+}
