@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: grdrotater.c,v 1.30 2009/01/09 04:02:35 guru Exp $
+ *	$Id: grdrotater.c,v 1.34 2009/06/12 02:04:46 remko Exp $
  *
  *   Copyright (c) 1999-2009 by P. Wessel
  *
@@ -65,10 +65,12 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'R':
 				case 'V':
 				case ':':
 				case 'b':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -88,9 +90,6 @@ int main (int argc, char **argv)
 				case 'G':
 					outfile = &argv[i][2];
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					outline = FALSE;
 					break;
@@ -131,8 +130,8 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdrotater %s - Finite rotation reconstruction of geographic grid\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdrotater <grdfile> -T<plon>/<plat>/<pomega> -G<rotgrid> [-F<polygonfile>]\n");
-		fprintf (stderr, "\t[%s] [%s] [-N] [-Q[b|c|l|n][[/]<threshold>]] [%s] [-S] [-V] [%s]\n", GMT_H_OPT, GMT_M_OPT, GMT_Rgeo_OPT, GMT_t_OPT);
-		fprintf (stderr, "\t[%s] > projpol\n\n", GMT_b_OPT);
+		fprintf (stderr, "\t[%s] [-N] [-Q[b|c|l|n][[/]<threshold>]] [%s] [-S] [-V] [%s]\n", GMT_H_OPT, GMT_Rgeo_OPT, GMT_t_OPT);
+		fprintf (stderr, "\t[%s] [%s] > projpol\n\n", GMT_b_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -144,7 +143,6 @@ int main (int argc, char **argv)
 		fprintf (stderr,"\t-F specifies a multi-segment closed polygon file that describes the area of the grid\n");
 		fprintf (stderr,"\t   that should be projected [Default projects entire grid]\n");
 		GMT_explain_option ('H');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N Do not output the rotated polygon or grid outline\n");
 		fprintf (stderr, "\t-Q Quick mode, use bilinear rather than bicubic [Default] interpolation.\n");
 		fprintf (stderr, "\t   Alternatively, select interpolation mode by adding b = B-spline, c = bicubic,\n");
@@ -161,6 +159,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('o');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -218,7 +217,7 @@ int main (int argc, char **argv)
 
 		/* Determine the wesn to be used to read the grdfile; or exit if file is outside -R */
 
-		if (GMT_grd_setregion (&g_head, &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+		if (GMT_grd_setregion (&g_head, &data_west, &data_east, &data_south, &data_north, interpolant)) {
 			fprintf (stderr, "%s: No grid values inside selected region - aborting\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
@@ -268,7 +267,7 @@ int main (int argc, char **argv)
 		if (outline) GMT_write_segmentheader (GMT_stdout, 2);
 		wp360 = wp180 = DBL_MAX;	ep360 = ep180 = -DBL_MAX;	spol = DBL_MAX;	npol = -DBL_MAX;
 		for (j = 0; j < pol->segment[i]->n_rows; j++) {
-			GMT_geo_to_cart (&pol->segment[i]->coord[GMT_Y][j], &pol->segment[i]->coord[GMT_X][j], P_original, TRUE);	/* Convert to a Cartesian x,y,z vector; TRUE since we have degrees */
+			GMT_geo_to_cart (pol->segment[i]->coord[GMT_Y][j], pol->segment[i]->coord[GMT_X][j], P_original, TRUE);	/* Convert to a Cartesian x,y,z vector; TRUE since we have degrees */
 			spotter_matrix_vect_mult (R, P_original, P_rotated);				/* Rotate the vector */
 			GMT_cart_to_geo (&pol->segment[i]->coord[GMT_Y][j], &pol->segment[i]->coord[GMT_X][j], P_rotated, TRUE);	/* Recover lon lat representation; TRUE to get degrees */
 			if (pol->segment[i]->coord[GMT_Y][j] < south) south = pol->segment[i]->coord[GMT_Y][j];
@@ -366,8 +365,7 @@ int main (int argc, char **argv)
 			
 			/* Here we are inside; get the coordinates and rotate back to original grid coordinates */
 			
-			xx = grd_x[i];	yy = grd_y[j];
-			GMT_geo_to_cart (&yy, &xx, P_rotated, TRUE);	/* Convert degree lon,lat to a Cartesian x,y,z vector */
+			GMT_geo_to_cart (grd_y[j], grd_x[j], P_rotated, TRUE);	/* Convert degree lon,lat to a Cartesian x,y,z vector */
 			spotter_matrix_vect_mult (R, P_rotated, P_original);	/* Rotate the vector */
 			GMT_cart_to_geo (&yy, &xx, P_original, TRUE);	/* Recover degree lon lat representation */
 			xx -= 360.0;
@@ -397,8 +395,7 @@ int main (int argc, char **argv)
 					if (iii < 0 || iii >= r_head.nx) continue;
 					ij = jjj * r_head.nx + iii;
 					if (!GMT_is_fnan (rotated_grid[ij])) continue;	/* Already done this */
-					xx = grd_x[iii];	yy = grd_y[jjj];
-					GMT_geo_to_cart (&yy, &xx, P_rotated, TRUE);	/* Convert degree lon,lat to a Cartesian x,y,z vector */
+					GMT_geo_to_cart (grd_y[jjj], grd_x[iii], P_rotated, TRUE);	/* Convert degree lon,lat to a Cartesian x,y,z vector */
 					spotter_matrix_vect_mult (R, P_rotated, P_original);	/* Rotate the vector */
 					GMT_cart_to_geo (&xx, &yy, P_original, TRUE);	/* Recover degree lon lat representation */
 					ii_o = GMT_x_to_i (xx, g_head.x_min, g_head.x_inc, g_head.xy_off, g_head.nx);
